(* Lunar Tower

   MiniLD #8 Mash-Up

   Copyright (c) 2009 Freeze.Dev (Johannes Stein)

  Permission is hereby granted, free of charge, to any person
  obtaining a copy of this software and associated documentation
  files (the "Software"), to deal in the Software without
  restriction, including without limitation the rights to use,
  copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be
  included in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
  OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
  WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
  OTHER DEALINGS IN THE SOFTWARE.

  info@freeze-dev.de
  www.freeze-dev.de

 V1.01 (09-03-09)  Game starts in windowed mode
                   Show FPS by pressing 'F'
                   Initialize Surface with 32 bits instead originally 16 bits
                   Tried to improve the movement of the letters

 V1.0  (09-03-08)  Initial release

 *)

program lunartower; // The name of this program

{$IFDEF DARWIN}
  {$linklib gcc}
  {$linklib SDLmain}
{$ENDIF}


{$IFDEF FPC}
  {$MODE DELPHI}
{$ENDIF}

uses
  ElysionLegacy, SysUtils, SDL;

const
  WIDTH = 800;
  HEIGHT = 600;
  BITS = 32;
  MAX_SPRITES = 7;

var
  Surface: TelSurface; // You have to declare a surface
  Timer: array[0..4] of TelTimer;
  Sprite: array[0..MAX_SPRITES] of TelSprite;
  Rects: array of TelRect;
  Font: TelFont;
  i: Integer;
  Fullscreen: Boolean = false;
  DebugInfo: Boolean;

  Credits, Damage, Life, UpDamage, Kills: Integer;

procedure LoadImages();
var j: ShortInt;
begin
  Sprite[0].LoadFromFile('../media/cursor.bmp');
  Sprite[0].SetTransparency(makeCol(255, 255, 255));
  Sprite[0].SetName('Cursor');

  Sprite[1].LoadFromFile('../media/background.jpg');
  Sprite[1].SetName('Background');

  Sprite[2].LoadFromFile('../media/tower.bmp');
  Sprite[2].SetTransparency(makeP2D(0, 0));
  Sprite[2].SetName('Tower');

  for j := 3 to 7 do
  begin
    Sprite[j].LoadFromFile('../media/enemy.bmp');
    Sprite[j].SetTransparency(makeP2D(0, 0));
  end;

end;

procedure SetVariables();
var j: ShortInt;
begin
  Credits := 0;
  Damage := 11;
  Life := 100;
  UpDamage := 15;
  Kills := 0;
  DebugInfo := false;

  Sprite[2].SetAlpha(255);

  for j := 3 to 7 do
  begin
    Sprite[j].SetName(IntToStr(100));
    Sprite[j].SetAlpha(255);
    Sprite[j].SetPosition(makeP2D((j - 3) * 40, 475));
  end;

  Timer[0].SetInterval(12);
  Timer[1].SetInterval(10);
  Timer[2].SetInterval(10);
  Timer[3].SetInterval(9);
  Timer[4].SetInterval(7);

  SDL_UpdateRect(Surface.SDL_Surface, 0, 0, 0, 0);
end;

begin
  Randomize;

  Surface := TelSurface.Create; // This surface must be created
  Font := TelFont.Create;

  // Create sprites
  for i := 0 to High(Sprite) do Sprite[i] := TelSprite.Create;

  // Initialize: 800 x 600 px @ 32 bit, windowed mode
  // Press w to switch between windowed and fullscreen mode
  Surface.Initialize(WIDTH, HEIGHT, BITS, Fullscreen);

  // Assign those sprites to the surface
  for i := 0 to High(Sprite) do Sprite[i].Assign(Surface);

  for i := 0 to High(Timer) do
  begin
    Timer[i] := TelTimer.Create;
    Timer[i].SetActive(true);
  end;

  //Sprite.Assign(Surface);
  Font.Assign(Surface);
  //Timer.SetActive(true);

  LoadImages; // Self-explanatory
  SetVariables;

  // Load font
  Font.LoadFromFile('../media/font.bmp');

  Surface.SetBackgroundImage(Sprite[1].GetSDLSurface);
  Sprite[2].SetPosition(makeP2D(600, 240));

  Surface.SetCaption('Lunar Tower (Mini LD 8)');

  // Setting the game loop
  while Surface.Run do
  begin
    Surface.BeginScene;
      Sprite[0].SetPosition(makeP2D(Surface.Cursor.X - 12, Surface.Cursor.Y - 12));

      for i := 3 to 7 do
      begin
        Sprite[i].SetAlpha(Round((StrToInt(Sprite[i].GetName)*255)/100));

        if Timer[i - 3].SingleAction then
        begin
          if Sprite[i].GetPosition.X <= Sprite[2].GetPosition.X - Sprite[3].GetWidth then
            Sprite[i].Move(makeP2D(Random(4) + 3, 0))
          else if Life > 0 then Life := Life - 1;
        end;
      end;

      if DebugInfo then
      begin
        Font.TextOut(0, 0, 'FPS: '+IntToStr(Surface.GetFPS), taLeft);
        Font.TextOut(0, Surface.SDL_Surface.h - 18, 'V1.01; (C) 2009 Freeze.Dev (Johannes Stein); http://www.freeze-dev.de', taLeft);
      end;

      Font.TextOut(0, 0, 'Level: '+IntToStr(Kills div 5 + 1), taRight);
      Font.TextOut(0, 20, 'Credits: '+IntToStr(Credits), taRight);
      Font.TextOut(0, 40, 'Health: '+IntToStr(Life)+'/100', taRight);
      Font.TextOut(0, 60, 'Kills: '+IntToStr(Kills), taRight);
      Font.TextOut(0, 0, 'Kill (by clicking on them) the evil letters', taCenter);
      Font.TextOut(0, 20, 'before they reach your precious tower.', taCenter);
      Font.TextOut(0, 50, 'Press ESC to quit', taCenter);

      Font.TextOut(600, 180, 'Upgrade Damage ['+IntToStr(UpDamage)+' Cr.]');

      if Life <= 0 then
      begin
        Font.TextOut(0, (HEIGHT - Font.GetHeight) div 2, 'Sorry, you lose. Press SPACE to try again.', taCenter);
        if Surface.KeyUp[K_SPACE] then SetVariables();
      end;

      if Surface.KeyDown[K_ESCAPE] then Surface.Quit;
      if Surface.KeyUp[K_d] or Surface.KeyUp[K_f] then DebugInfo := not DebugInfo;
      if Surface.KeyUp[K_w] then
      begin
        Fullscreen := not Fullscreen;
        Surface.SetVideoMode(WIDTH, HEIGHT, BITS, Fullscreen);
      end;

      if Surface.MouseButtonUp[BUTTON_LEFT] then
      begin
        if Surface.OnPoint(Surface.Cursor, makeRect(590, 170, 200, 40)) then
        begin
          if Credits >= UpDamage then
          begin
            Damage := Round(Damage * 1.25);
            UpDamage := UpDamage * 2;
          end;
        end;

        for i := 3 to High(Sprite) do
        begin
          if Sprite[i].OnPoint(Surface.Cursor) then
          begin
            if (StrToInt(Sprite[i].GetName) - Damage) <= 0 then
            begin
              Kills := Kills + 1;

              if Kills > 0 then
              begin
                if (Kills mod 5) = 0 then
                begin
                  if Timer[i - 3].GetInterval > 1 then Timer[i - 3].SetInterval(Timer[i - 3].GetInterval - 1);
                end;
              end;

              Credits := Credits + 1 + (Kills div 5);

              Sprite[i].SetPosition(makeP2D(0, 475));
              Sprite[i].SetName(IntToStr(100));
              Sprite[i].SetAlpha(255);
            end else Sprite[i].SetName(IntToStr(StrToInt(Sprite[i].GetName) - Damage));
          end;
        end;
      end;

      Sprite[2].SetAlpha(Round((Life * 255) / 100));
      Sprite[2].Draw;

      for i := 3 to 7 do Sprite[i].Draw;
      Sprite[0].Draw;

      for i := 0 to MAX_SPRITES do
      begin
        setLength(Rects, MAX_SPRITES);
        Rects[i] := makeRect(Sprite[i].GetPosition.X, Sprite[i].GetPosition.Y, Sprite[i].GetWidth, Sprite[i].GetHeight);
      end;

    Surface.EndScene(Rects);
  end;

  Font.Free;
  for i := 0 to High(Timer) do Timer[i].Free;
  for i := 0 to High(Sprite) do Sprite[i].Free;
  Surface.Finalize; // Finally, finalize the surface
end.
